require_relative 'player'
require_relative 'die'
require_relative 'game_turn'
require_relative 'treasure_trove'

module StudioGame
  player1 = Player.new('moe')
  player2 = Player.new('larry', 60)
  player3 = Player.new('curly', 125)

  class Game
    attr_reader :name

    def initialize(title)
      @players = []
      @title = title
    end

    def add_player(player)
      @players << player
    end

    def play(rounds)
      puts "There are #{@players.size} players in #{@name}:"
      treasures = TreasureTrove::TREASURES

      1.upto(rounds) do |round|
        puts "Round #{round}"
        @players.each do |player|
          GameTurn.take_turn(player)
          puts player
        end
      end

      puts "\nThere are #{treasures.size} treasures to be found:"

      treasures.each do |treasure|
        puts "A #{treasure.name} is worth #{treasure.points} points"
      end
    end

    def print_name_and_health(player)
      puts "#{player.name} #{player.health}"
    end

    def total_points
      @players.each.reduce(0) { |acc, player| acc + player.points }
    end

    def print_stats
      strong, wimpy = @players.partition(&:strong?)

      puts "\n#{@title} Statistics:"

      puts "\n#{strong.size} strong players:"
      strong.each do |s|
        print_name_and_health(s)
      end

      puts "\n#{wimpy.size} wimpy players:"
      wimpy.each do |w|
        print_name_and_health(w)
      end

      puts 'Knuckleheads High Scores:'
      @players.sort.each do |p|
        puts high_score_entry(p)
      end

      puts "\n#{total_points} total points from treasures found"

      @players.sort.each do |player|
        puts "\n#{player.name}'s point totals:"
        player.each_found_treasure do |treasure|
          puts "#{treasure.points} total #{treasure.name} points"
        end
        puts "#{player.points} grand total points"
      end
    end

    def load_players(file)
      File.readlines(file).each do |line|
        add_player(Player.from_csv(line))
      end
    end

    def save_high_scores(to_file = 'high_scores.txt')
      File.open(to_file, 'w') do |file|
        file.puts "#{@title} High Scores:"
        @players.sort.each do |p|
          file.puts high_score_entry(p)
        end
      end
    end

    def high_score_entry(player)
      formatted_name = player.name.ljust(20, '.')
      "#{formatted_name} #{player.score}"
    end
  end
end

if __FILE__ == $0
  knuckleheads = Game.new('Knuckleheads')
  knuckleheads.add_player(player1)
  knuckleheads.add_player(player2)
  knuckleheads.add_player(player3)
  knuckleheads.play(3)
end
